// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KDB_VERSION_H
#define KDB_VERSION_H

#define KDB_VERSION_STRING "3.0.2"
#define KDB_VERSION_MAJOR 3
#define KDB_VERSION_MINOR 0
#define KDB_VERSION_PATCH 2
#define KDB_VERSION ((3<<16)|(0<<8)|(2))

#endif
